@@ -24,30 +24,12 @@
 
 ;;; Code:
 
-(require 'lsp)
+(require 'lsp-mode)
 (require 'dash)
 (require 'dash-functional)
 (require 'rx)
-(require 'lsp-pyls)
-(require 'lsp-rust)
-(require 'lsp-solargraph)
-(require 'lsp-vetur)
-(require 'lsp-intelephense)
-(require 'lsp-css)
-(require 'lsp-xml)
-(require 'lsp-go)
-(require 'lsp-clojure)
-(require 'lsp-dart)
-(require 'lsp-elm)
-(require 'lsp-metals)
-(require 'lsp-fsharp)
-(require 'lsp-erlang)
-(require 'lsp-haxe)
-(require 'lsp-vhdl)
-(require 'lsp-yaml)
-(require 'lsp-terraform)
-(require 'lsp-pwsh)
-(require 'lsp-csharp)
+(require 'cl-lib)
+(require 'em-glob)
 
 ;;; Ada
 (defgroup lsp-ada nil
@@ -113,35 +95,51 @@ See instructions at https://marketplace.visualstudio.com/items?itemName=mads-har
   :group 'lsp-bash
   :package-version '(lsp-mode . "6.2"))
 
+(defcustom lsp-bash-glob-pattern nil
+  "Glob pattern used to find shell script files to parse."
+  :type 'string
+  :group 'lsp-bash
+  :package-version '(lsp-mode . "6.3"))
+
+(defun lsp-bash--bash-ls-server-command ()
+  "Startup command for Bash language server."
+  (list (lsp-package-path 'bash-language-server) "start"))
+
+(lsp-dependency 'bash-language-server
+                '(:system "bash-language-server")
+                '(:npm :package "bash-language-server"
+                       :path "bash-language-server"))
+
 (lsp-register-client
- (make-lsp-client :new-connection (lsp-stdio-connection '("bash-language-server" "start"))
-                  :major-modes '(sh-mode)
-                  :priority -1
-                  :environment-fn (lambda ()
-                                    '(("EXPLAINSHELL_ENDPOINT" . lsp-bash-explainshell-endpoint)
-                                      ("HIGHLIGHT_PARSING_ERRORS" . lsp-bash-highlight-parsing-errors)))
-                  :server-id 'bash-ls))
+ (make-lsp-client
+  :new-connection (lsp-stdio-connection #'lsp-bash--bash-ls-server-command)
+  :major-modes '(sh-mode)
+  :priority -1
+  :environment-fn (lambda ()
+                    '(("EXPLAINSHELL_ENDPOINT" . lsp-bash-explainshell-endpoint)
+                      ("HIGHLIGHT_PARSING_ERRORS" . lsp-bash-highlight-parsing-errors)
+                      ("GLOB_PATTERN" . lsp-bash-glob-pattern)))
+  :server-id 'bash-ls
+  :download-server-fn (lambda (_client callback error-callback _update?)
+                        (lsp-package-ensure 'bash-language-server callback error-callback))))
+
 
 ;;; Groovy
 (defgroup lsp-groovy nil
   "LSP support for Groovy, using groovy-language-server"
   :group 'lsp-mode
-  :link '(url-link "https://github.com/palantir/language-servers"))
+  :link '(url-link "https://github.com/prominic/groovy-language-server"))
 
-(defcustom lsp-groovy-server-install-dir
-  (locate-user-emacs-file "groovy-language-server/")
-  "Install directory for groovy-language-server.
-A slash is expected at the end.
-This directory shoud contain a file matching groovy-language-server-*.jar"
+(defcustom lsp-groovy-server-file
+  (locate-user-emacs-file "groovy-language-server/groovy-language-server-all.jar")
+  "JAR file path for groovy-language-server-all.jar."
   :group 'lsp-groovy
   :risky t
-  :type 'directory)
+  :type 'file)
 
 (defun lsp-groovy--lsp-command ()
   "Generate LSP startup command."
-  `("java"
-    "-cp" ,(concat (file-truename lsp-groovy-server-install-dir) "*")
-    "com.palantir.ls.groovy.GroovyLanguageServer"))
+  `("java" "-jar" ,(expand-file-name lsp-groovy-server-file)))
 
 (lsp-register-client
  (make-lsp-client :new-connection (lsp-stdio-connection 'lsp-groovy--lsp-command)
@@ -150,19 +148,17 @@ This directory shoud contain a file matching groovy-language-server-*.jar"
                   :server-id 'groovy-ls))
 
 ;;; TypeScript/JavaScript
+
+(lsp-dependency 'javascript-typescript-langserver
+                '(:system "javascript-typescript-stdio")
+                '(:npm :package "javascript-typescript-langserver"
+                       :path "javascript-typescript-stdio"))
+
 (defgroup lsp-typescript-javascript nil
   "Support for TypeScript/JavaScript, using Sourcegraph's JavaScript/TypeScript language server."
   :group 'lsp-mode
   :link '(url-link "https://github.com/sourcegraph/javascript-typescript-langserver"))
 
-(defcustom lsp-clients-javascript-typescript-server "javascript-typescript-stdio"
-  "The javascript-typescript-stdio executable to use.
-Leave as just the executable name to use the default behavior of
-finding the executable with variable `exec-path'."
-  :group 'lsp-typescript-javascript
-  :risky t
-  :type 'file)
-
 (defcustom lsp-clients-typescript-javascript-server-args '()
   "Extra arguments for the typescript-language-server language server."
   :group 'lsp-typescript-javascript
@@ -172,16 +168,22 @@ finding the executable with variable `exec-path'."
 (defun lsp-typescript-javascript-tsx-jsx-activate-p (filename &optional _)
   "Check if the javascript-typescript language server should be enabled based on FILENAME."
   (or (string-match-p (rx (one-or-more anything) "." (or "ts" "js") (opt "x") string-end) filename)
-      (derived-mode-p 'js-mode 'js2-mode 'typescript-mode)))
+      (and (derived-mode-p 'js-mode 'js2-mode 'typescript-mode)
+           (not (derived-mode-p 'json-mode)))))
 
 (lsp-register-client
  (make-lsp-client :new-connection (lsp-stdio-connection (lambda ()
-                                                          (cons lsp-clients-javascript-typescript-server
+                                                          (cons (lsp-package-path 'javascript-typescript-langserver)
                                                                 lsp-clients-typescript-javascript-server-args)))
                   :activation-fn 'lsp-typescript-javascript-tsx-jsx-activate-p
                   :priority -3
-                  :ignore-messages '("readFile .*? requested by TypeScript but content not available")
-                  :server-id 'jsts-ls))
+                  :completion-in-comments? t
+                  :server-id 'jsts-ls
+                  :download-server-fn (lambda (_client callback error-callback _update?)
+                                        (lsp-package-ensure
+                                         'javascript-typescript-langserver
+                                         callback
+                                         error-callback))))
 
 
 ;;; TypeScript
@@ -190,14 +192,6 @@ finding the executable with variable `exec-path'."
   :group 'lsp-mode
   :link '(url-link "https://github.com/theia-ide/typescript-language-server"))
 
-(defcustom lsp-clients-typescript-server "typescript-language-server"
-  "The typescript-language-server executable to use.
-Leave as just the executable name to use the default behavior of
-finding the executable with variable `exec-path'."
-  :group 'lsp-typescript
-  :risky t
-  :type 'file)
-
 (defcustom lsp-clients-typescript-server-args '("--stdio")
   "Extra arguments for the typescript-language-server language server."
   :group 'lsp-typescript
@@ -226,18 +220,39 @@ directory containing the package. Example:
                                                   (and name location))
                                                 xs)))))
 
+(lsp-dependency 'typescript-language-server
+                '(:system "typescript-language-server")
+                '(:npm :package "typescript-language-server"
+                       :path "typescript-language-server"))
+
+(lsp-dependency 'typescript
+                '(:system "tsserver")
+                '  (:npm :package "typescript"
+                         :path "tsserver"))
+
 (lsp-register-client
  (make-lsp-client :new-connection (lsp-stdio-connection (lambda ()
-                                                          (cons lsp-clients-typescript-server
-                                                                lsp-clients-typescript-server-args)))
+                                                          `(,(lsp-package-path 'typescript-language-server)
+                                                            "--tsserver-path"
+                                                            ,(lsp-package-path 'typescript)
+                                                            ,@lsp-clients-typescript-server-args)))
                   :activation-fn 'lsp-typescript-javascript-tsx-jsx-activate-p
                   :priority -2
                   :completion-in-comments? t
                   :initialization-options (lambda ()
                                             (list :plugins lsp-clients-typescript-plugins
-                                                  :logVerbosity lsp-clients-typescript-log-verbosity))
+                                                  :logVerbosity lsp-clients-typescript-log-verbosity
+                                                  :tsServerPath (lsp-package-path 'typescript)))
                   :ignore-messages '("readFile .*? requested by TypeScript but content not available")
-                  :server-id 'ts-ls))
+                  :server-id 'ts-ls
+                  :download-server-fn (lambda (_client callback error-callback _update?)
+                                        (lsp-package-ensure
+                                         'typescript
+                                         (-partial #'lsp-package-ensure
+                                                   'typescript-language-server
+                                                   callback
+                                                   error-callback)
+                                         error-callback))))
 
 
 
@@ -264,40 +279,41 @@ finding the executable with variable `exec-path'."
 (defun lsp-clients-flow-tag-file-present-p (file-name)
   "Check if the '// @flow' or `/* @flow */' tag is present in
 the contents of FILE-NAME."
-  (with-temp-buffer
-    (insert-file-contents file-name)
-    (lsp-clients-flow-tag-string-present-p (buffer-string))))
+  (if-let ((buffer (find-buffer-visiting file-name)))
+      (with-current-buffer buffer
+        (lsp-clients-flow-tag-string-present-p))
+    (with-temp-buffer
+      (insert-file-contents file-name)
+      (lsp-clients-flow-tag-string-present-p))))
 
-(defun lsp-clients-flow-tag-string-present-p (file-contents)
+(defun lsp-clients-flow-tag-string-present-p ()
   "Helper for `lsp-clients-flow-tag-file-present-p' that works
 with the file contents."
-  (with-temp-buffer
-    (insert file-contents)
-    (save-excursion
-      (goto-char (point-min))
-      (let (stop found)
-        (while (not stop)
-          (when (not (re-search-forward "[^\n[:space:]]" nil t))
-            (setq stop t))
-          (if (equal (point) (point-min))
-              (setq stop t)
-            (backward-char))
-          (cond ((or (looking-at "//+[ ]*@flow")
-                     (looking-at "/\\**[ ]*@flow")
-                     (looking-at "[ ]*\\*[ ]*@flow"))
-                 (setq found t)
-                 (setq stop t))
-                ((looking-at "//")
-                 (forward-line))
-                ((looking-at "*")
-                 (forward-line))
-                ((looking-at "/\\*")
-                 (save-excursion
-                   (when (not (re-search-forward "*/" nil t))
-                     (setq stop t)))
-                 (forward-line))
-                (t (setq stop t))))
-        found))))
+  (save-excursion
+    (goto-char (point-min))
+    (let (stop found)
+      (while (not stop)
+        (when (not (re-search-forward "[^\n[:space:]]" nil t))
+          (setq stop t))
+        (if (equal (point) (point-min))
+            (setq stop t)
+          (backward-char))
+        (cond ((or (looking-at "//+[ ]*@flow")
+                   (looking-at "/\\**[ ]*@flow")
+                   (looking-at "[ ]*\\*[ ]*@flow"))
+               (setq found t)
+               (setq stop t))
+              ((looking-at "//")
+               (forward-line))
+              ((looking-at "*")
+               (forward-line))
+              ((looking-at "/\\*")
+               (save-excursion
+                 (when (not (re-search-forward "*/" nil t))
+                   (setq stop t)))
+               (forward-line))
+              (t (setq stop t))))
+      found)))
 
 (defun lsp-clients-flow-project-p (file-name)
   "Check if FILE-NAME is part of a Flow project, that is, if
@@ -308,8 +324,8 @@ there is a .flowconfig file in the folder hierarchy."
   "Check if the Flow language server should be enabled for a
 particular FILE-NAME and MODE."
   (and (derived-mode-p 'js-mode 'web-mode 'js2-mode 'flow-js2-mode 'rjsx-mode)
-       (lsp-clients-flow-project-p file-name)
-       (lsp-clients-flow-tag-file-present-p file-name)))
+       (or (lsp-clients-flow-project-p file-name)
+	   (lsp-clients-flow-tag-file-present-p file-name))))
 
 (lsp-register-client
  (make-lsp-client :new-connection (lsp-stdio-connection (lambda ()
@@ -334,24 +350,23 @@ particular FILE-NAME and MODE."
 
 (defun lsp-php--create-connection ()
   "Create lsp connection."
-  (plist-put
-   (lsp-stdio-connection
-    (lambda () lsp-clients-php-server-command))
-   :test? (lambda ()
-            (if (and (cdr lsp-clients-php-server-command)
-                     (eq (string-match-p "php[0-9.]*\\'" (car lsp-clients-php-server-command)) 0))
-                ;; Start with the php command and the list has more elems. Test the existence of the PHP script.
-                (let ((php-file (nth 1 lsp-clients-php-server-command)))
-                  (or (file-exists-p php-file)
-                      (progn
-                        (lsp-log "%s is not present." php-file)
-                        nil)))
-              t))))
+  (lsp-stdio-connection
+   (lambda () lsp-clients-php-server-command)
+   (lambda ()
+     (if (and (cdr lsp-clients-php-server-command)
+              (eq (string-match-p "php[0-9.]*\\'" (car lsp-clients-php-server-command)) 0))
+         ;; Start with the php command and the list has more elems. Test the existence of the PHP script.
+         (let ((php-file (nth 1 lsp-clients-php-server-command)))
+           (or (file-exists-p php-file)
+               (progn
+                 (lsp-log "%s is not present." php-file)
+                 nil)))
+       t))))
 
 (lsp-register-client
  (make-lsp-client :new-connection (lsp-php--create-connection)
                   :major-modes '(php-mode)
-                  :priority -2
+                  :priority -3
                   :server-id 'php-ls))
 
 
@@ -382,13 +397,16 @@ particular FILE-NAME and MODE."
                   :priority -1
                   :server-id 'ocaml-ls))
 
-(defgroup lsp-merlin nil
-  "LSP support for OCaml, using merlin."
+(defgroup lsp-ocaml-lsp-server nil
+  "LSP support for OCaml, using ocaml-lsp-server."
   :group 'lsp-mode
-  :link '(url-link "https://github.com/ocaml/merlin"))
+  :link '(url-link "https://github.com/ocaml/ocaml-lsp"))
 
-(defcustom lsp-merlin-command
-  '("ocamlmerlin-lsp")
+(define-obsolete-variable-alias 'lsp-merlin 'lsp-ocaml-lsp-server)
+(define-obsolete-variable-alias 'lsp-merlin-command 'lsp-ocaml-lsp-server-command)
+
+(defcustom lsp-ocaml-lsp-server-command
+  '("ocamllsp")
   "Command to start ocaml-language-server."
   :group 'lsp-ocaml
   :type '(choice
@@ -399,10 +417,10 @@ particular FILE-NAME and MODE."
 (lsp-register-client
  (make-lsp-client
   :new-connection
-  (lsp-stdio-connection (lambda () lsp-merlin-command))
+  (lsp-stdio-connection (lambda () lsp-ocaml-lsp-server-command))
   :major-modes '(caml-mode tuareg-mode)
   :priority 0
-  :server-id 'merlin))
+  :server-id 'ocaml-lsp-server))
 
 
 ;; C-family (C, C++, Objective-C, Objective-C++)
@@ -437,18 +455,33 @@ finding the executable with `exec-path'."
                   :priority -1
                   :server-id 'clangd))
 
-(defun lsp-clients-register-clangd ()
-  (warn "This call is no longer needed. clangd is now automatically registered. Delete lsp-clients-register-clangd call from your config."))
-
-(make-obsolete 'lsp-clients-register-clangd
-               "This function is no longer needed, as clangd is now automatically registered."
-               "lsp-mode 6.1")
+(cl-defmethod lsp-clients-extract-signature-on-hover (contents (_server-id (eql clangd)))
+  "Extract a representative line from clangd's CONTENTS, to show in the echo area.
+This function tries to extract the type signature from CONTENTS,
+or the first line if it cannot do so. A single line is always
+returned to avoid that the echo area grows uncomfortably."
+  (with-temp-buffer
+    (-let [value (lsp:markup-content-value contents)]
+      (insert value)
+      (goto-char (point-min))
+      (if (re-search-forward (rx (seq "```cpp\n"
+                                      (opt (group "//"
+                                                  (zero-or-more nonl)
+                                                  "\n"))
+                                      (group
+                                       (one-or-more
+                                        (not (any "`")))
+                                       "\n")
+                                      "```")) nil t nil)
+          (progn (narrow-to-region (match-beginning 2) (match-end 2))
+                 (lsp--render-element (lsp-join-region (point-min) (point-max))))
+        (car (s-lines (lsp--render-element contents)))))))
 
 ;; Elixir
 (defgroup lsp-elixir nil
   "LSP support for Elixir, using elixir-ls."
   :group 'lsp-mode
-  :link '(url-link "https://github.com/JakeBecker/elixir-ls"))
+  :link '(url-link "https://github.com/elixir-lsp/elixir-ls"))
 
 (defcustom lsp-clients-elixir-server-executable
   (if (equal system-type 'windows-nt)
@@ -511,11 +544,17 @@ finding the executable with `exec-path'."
   :group 'lsp-mode
   :link '(url-link "https://github.com/fwcd/KotlinLanguageServer"))
 
-(defcustom lsp-kotlin-language-server-path ""
-  "Optionally a custom path to the language server executable."
+(define-obsolete-variable-alias
+  'lsp-kotlin-language-server-path
+  'lsp-clients-kotlin-server-executable
+  "lsp-mode 6.4")
+
+(defcustom lsp-clients-kotlin-server-executable "kotlin-language-server"
+  "The kotlin-language-server executable to use.
+Leave as just the executable name to use the default behavior of finding the
+executable with `exec-path'."
   :type 'string
-  :group 'lsp-kotlin
-  :package-version '(lsp-mode . "6.1"))
+  :group 'lsp-kotlin)
 
 (defcustom lsp-kotlin-trace-server "off"
   "Traces the communication between VSCode and the Kotlin language server."
@@ -531,7 +570,7 @@ finding the executable with `exec-path'."
 
 (defcustom lsp-kotlin-linting-debounce-time 250
   "[DEBUG] Specifies the debounce time limit. Lower to increase
-responsiveness at the cost of possibile stability issues."
+responsiveness at the cost of possible stability issues."
   :type 'number
   :group 'lsp-kotlin
   :package-version '(lsp-mode . "6.1"))
@@ -573,11 +612,11 @@ responsiveness at the cost of possibile stability issues."
    ("kotlin.linting.debounceTime" lsp-kotlin-linting-debounce-time)
    ("kotlin.compiler.jvm.target" lsp-kotlin-compiler-jvm-target)
    ("kotlin.trace.server" lsp-kotlin-trace-server)
-   ("kotlin.languageServer.path" lsp-kotlin-language-server-path)))
+   ("kotlin.languageServer.path" lsp-clients-kotlin-server-executable)))
 
 (lsp-register-client
  (make-lsp-client
-  :new-connection (lsp-stdio-connection '("kotlin-language-server"))
+  :new-connection (lsp-stdio-connection lsp-clients-kotlin-server-executable)
   :major-modes '(kotlin-mode)
   :priority -1
   :server-id 'kotlin-ls
@@ -629,7 +668,13 @@ responsiveness at the cost of possibile stability issues."
 
 ;;; Angular
 (defcustom lsp-clients-angular-language-server-command
-  `("node"  ,(expand-file-name "~/.angular/extension/server/server.js") "--stdio")
+  '("node"
+    "/usr/lib/node_modules/@angular/language-server"
+    "--ngProbeLocations"
+    "/usr/lib/node_modules"
+    "--tsProbeLocations"
+    "/usr/lib/node_modules"
+    "--stdio")
   "The command that starts the angular language server."
   :group 'lsp-clients-angular
   :type '(choice
@@ -637,23 +682,38 @@ responsiveness at the cost of possibile stability issues."
           (repeat :tag "List of string values"
                   string)))
 
+(defun lsp-client--angular-start-loading (_workspace params)
+  (lsp--info "Started loading project %s" params))
+
+(defun lsp-client--angular-finished-loading (_workspace params)
+  (lsp--info "Finished loading project %s" params))
+
 (lsp-register-client
  (make-lsp-client :new-connection (lsp-stdio-connection
                                    (lambda () lsp-clients-angular-language-server-command))
                   :activation-fn (lambda (&rest _args)
-                                   (and (string-match-p ".*\.html$" (buffer-file-name))
+                                   (and (string-match-p "\\.html\\'" (buffer-file-name))
                                         (lsp-workspace-root)
                                         (file-exists-p (f-join (lsp-workspace-root) "angular.json"))))
                   :priority -1
+                  :notification-handlers (ht ("angular-language-service/projectLoadingStart" #'lsp-client--angular-start-loading)
+                                             ("angular-language-service/projectLoadingFinish" #'lsp-client--angular-finished-loading))
                   :add-on? t
                   :server-id 'angular-ls))
 
 
 ;; TeX
 (defgroup lsp-tex nil
-  "LSP support for TeX and friends, using Digestif."
+  "LSP support for TeX and friends, using Digestif and texlab."
   :group 'lsp-mode
-  :link '(url-link "https://github.com/astoff/digestif/"))
+  :link '(url-link "https://github.com/astoff/digestif/")
+  :link '(url-link "https://github.com/latex-lsp/texlab"))
+
+(defcustom lsp-tex-server 'texlab
+  "Choose LSP tex server."
+  :type '(choice (const :tag "texlab" texlab)
+                 (const :tag "digestif" digestif))
+  :group 'lsp-tex)
 
 (defcustom lsp-clients-digestif-executable "digestif"
   "Command to start the Digestif language server."
@@ -664,17 +724,29 @@ responsiveness at the cost of possibile stability issues."
 (lsp-register-client
  (make-lsp-client :new-connection (lsp-stdio-connection lsp-clients-digestif-executable)
                   :major-modes '(plain-tex-mode latex-mode)
-                  :priority -1
+                  :priority (if (eq lsp-tex-server 'digestif) 1 -1)
                   :server-id 'digestif))
 
+(defcustom lsp-clients-texlab-executable "texlab"
+  "Command to start the texlab language server."
+  :group 'lsp-tex
+  :risky t
+  :type 'file)
+
+(lsp-register-client
+ (make-lsp-client :new-connection (lsp-stdio-connection lsp-clients-texlab-executable)
+                  :major-modes '(plain-tex-mode latex-mode)
+                  :priority (if (eq lsp-tex-server 'texlab) 1 -1)
+                  :server-id 'texlab))
+
 
 ;; Vim script
 (defgroup lsp-vim nil
-  "LSP support for TeX and friends, using Digestif."
+  "LSP support for viml using vim-language-server"
   :group 'lsp-mode)
 
 (defcustom lsp-clients-vim-executable '("vim-language-server" "--stdio")
-  "Command to start the Digestif language server."
+  "Command to start the vim language server."
   :group 'lsp-vim
   :risky t
   :type 'file)
@@ -721,12 +793,11 @@ responsiveness at the cost of possibile stability issues."
 
 (defun lsp-clients-emmy-lua--create-connection ()
   "Create connection to emmy lua language server."
-  (plist-put
-   (lsp-stdio-connection
-    (lambda ()
-      (list lsp-clients-emmy-lua-java-path "-jar" lsp-clients-emmy-lua-jar-path)))
-   :test? (lambda ()
-            (f-exists? lsp-clients-emmy-lua-jar-path))))
+  (lsp-stdio-connection
+   (lambda ()
+     (list lsp-clients-emmy-lua-java-path "-jar" lsp-clients-emmy-lua-jar-path))
+   (lambda ()
+     (f-exists? lsp-clients-emmy-lua-jar-path))))
 
 (lsp-register-client
  (make-lsp-client :new-connection (lsp-clients-emmy-lua--create-connection)
@@ -771,5 +842,192 @@ responsiveness at the cost of possibile stability issues."
                   :major-modes '(crystal-mode)
                   :server-id 'scry))
 
+
+;; Nim
+(defgroup lsp-nim nil
+  "LSP support for Nim, using nimlsp."
+  :group 'lsp-mode
+  :link '(url-link "https://github.com/PMunch/nimlsp"))
+
+(lsp-register-client
+ (make-lsp-client :new-connection (lsp-stdio-connection "nimlsp")
+                  :major-modes '(nim-mode)
+                  :priority -1
+                  :server-id 'nimls))
+
+;; Dhall
+(defgroup lsp-dhall nil
+  "LSP support for Dhall, using dhall-lsp-server."
+  :group 'lsp-mode
+  :link '(url-link "https://github.com/dhall-lang/dhall-haskell"))
+
+(lsp-register-client
+ (make-lsp-client :new-connection (lsp-stdio-connection "dhall-lsp-server")
+                  :major-modes '(dhall-mode)
+                  :priority -1
+                  :server-id 'dhallls))
+
+;; CMake
+(defgroup lsp-cmake nil
+  "LSP support for CMake, using cmake-language-server."
+  :group 'lsp-mode
+  :link '(url-link "https://github.com/regen100/cmake-language-server"))
+
+(lsp-register-client
+ (make-lsp-client :new-connection (lsp-stdio-connection "cmake-language-server")
+                  :major-modes '(cmake-mode)
+                  :priority -1
+                  :server-id 'cmakels))
+
+;; PureScript
+(defgroup lsp-purescript nil
+  "LSP support for PureScript, using purescript-language-server."
+  :group 'lsp-mode
+  :link '(url-link "https://github.com/nwolverson/purescript-language-server"))
+
+(defcustom lsp-purescript-server-executable nil
+  "Path to server executable."
+  :type 'string
+  :risky t
+  :group 'lsp-purescript)
+
+(defcustom lsp-purescript-server-args
+  '("--stdio")
+  "Arguments to pass to the server."
+  :type '(repeat string)
+  :risky t
+  :group 'lsp-purescript)
+
+(defun lsp-purescript--server-command ()
+  "Generate LSP startup command for purescript-language-server."
+  (cons (or lsp-purescript-server-executable
+            (lsp-package-path 'purescript-language-server))
+        lsp-purescript-server-args))
+
+(lsp-dependency 'purescript-language-server
+                '(:system "purescript-language-server")
+                '(:npm :package "purescript-language-server"
+                       :path "purescript-language-server"))
+
+(lsp-register-client
+ (make-lsp-client
+  :new-connection (lsp-stdio-connection
+                   #'lsp-purescript--server-command)
+  :major-modes '(purescript-mode)
+  :priority -1
+  :server-id 'pursls
+  :download-server-fn (lambda (_client callback error-callback _update?)
+                        (lsp-package-ensure 'purescript-language-server callback error-callback))))
+
+;;; Rf
+(defgroup lsp-rf nil
+  "Settings for Robot Framework Language Server."
+  :group 'lsp-mode
+  :tag "Language Server"
+  :link '(url-link "https://github.com/tomi/vscode-rf-language-server.git"))
+
+(defcustom lsp-rf-language-server-start-command '("~/.nvm/versions/node/v9.11.2/bin/node" "~/.vscode/extensions/tomiturtiainen.rf-intellisense-2.8.0/server/server.js")
+  "Path to the server.js file of the rf-intellisense server. Accepts a list of strings (path/to/interpreter path/to/server.js)"
+  :type 'list
+  :group 'lsp-rf)
+
+(defcustom lsp-rf-language-server-include-paths []
+  "An array of files that should be included by the parser. Glob patterns as strings are accepted (eg. *.robot between double quotes)"
+  :type 'lsp-string-vector
+  :group 'lsp-rf)
+
+(defcustom lsp-rf-language-server-exclude-paths []
+  "An array of files that should be ignored by the parser. Glob patterns as strings are accepted (eg. *bad.robot between double quotes)"
+  :type 'lsp-string-vector
+  :group 'lsp-rf)
+
+(defcustom lsp-rf-language-server-dir "~/.vscode/extensions/tomiturtiainen.rf-intellisense-2.8.0/server/library-docs/"
+  "Libraries directory for libraries in lsp-rf-language-server-libraries"
+  :type 'string
+  :group 'lsp-rf)
+
+(defcustom lsp-rf-language-server-libraries ["BuiltIn-3.1.1" "Collections-3.0.4"]
+  "Libraries whose keywords are suggested with auto-complete"
+  :type '(repeat string)
+  ;; :type 'lsp-string-vector
+  :group 'lsp-rf)
+
+(defcustom lsp-rf-language-server-log-level "debug"
+  "What language server log messages are printed"
+  :type 'string
+  ;; :type '(choice (:tag "off" "errors" "info" "debug"))
+  :group 'lsp-rf)
+
+(defcustom lsp-rf-language-server-trace-server "verbose"
+  "Traces the communication between VSCode and the rfLanguageServer service."
+  :type 'string
+  ;; :type '(choice (:tag "off" "messages" "verbose"))
+  :group 'lsp-rf)
+
+(defun parse-rf-language-server-library-dirs (dirs)
+  (vconcat (mapcar
+   (lambda (x)
+     (concat
+      (expand-file-name
+       lsp-rf-language-server-dir)
+      x
+      ".json"))
+   dirs)))
+
+(defun expand-start-command ()
+  (mapcar 'expand-file-name lsp-rf-language-server-start-command))
+
+(defun parse-rf-language-server-globs-to-regex (vector)
+  "Converts vector with globs to regex"
+    (concat "\\("(mapconcat 'eshell-glob-regexp vector "\\|") "\\)"))
+
+(defun parse-rf-language-server-include-path-regex (vector)
+  "Creates regexp to select files from workspace directory"
+  (let ((globs (if (eq vector [])
+                        ["*.robot" "*.resource"]
+                      vector)))
+    (parse-rf-language-server-globs-to-regex globs)))
+
+(defun parse-rf-language-server-exclude-paths (seq)
+  "Creates regexp to select files from workspace directory"
+  (if (eq lsp-rf-language-server-exclude-paths [])
+      seq
+  (cl-delete-if (lambda (x) (string-match-p
+                             (parse-rf-language-server-globs-to-regex
+                              lsp-rf-language-server-exclude-paths)
+                             x))
+                seq)))
+
+(lsp-register-custom-settings
+ '(
+   ("rfLanguageServer.trace.server" lsp-rf-language-server-trace-server)
+   ("rfLanguageServer.logLevel" lsp-rf-language-server-log-level)
+   ("rfLanguageServer.libraries" lsp-rf-language-server-libraries)
+   ("rfLanguageServer.excludePaths" lsp-rf-language-server-exclude-paths)
+   ("rfLanguageServer.includePaths" lsp-rf-language-server-include-paths)))
+
+(lsp-register-client
+ (make-lsp-client :new-connection (lsp-stdio-connection
+                                   (expand-start-command))
+                  :major-modes '(robot-mode)
+                  :server-id 'rf-intellisense
+                  ;; :library-folders-fn (lambda (_workspace)
+                  ;;                        lsp-rf-language-server-libraries)
+                  :library-folders-fn (lambda (_workspace)
+                                         (parse-rf-language-server-library-dirs
+                                         lsp-rf-language-server-libraries))
+                  :initialized-fn (lambda (workspace)
+                                    (with-lsp-workspace workspace
+                                      (lsp--set-configuration
+                                       (lsp-configuration-section "rfLanguageServer"))
+                                      (lsp-request "buildFromFiles"
+                                                   (list :files
+                                                         (vconcat
+                                                          (parse-rf-language-server-exclude-paths
+                                                           (directory-files-recursively
+                                                            (lsp--workspace-root workspace)
+                                                            (parse-rf-language-server-include-path-regex
+                                                             lsp-rf-language-server-include-paths))))))))))
+
 (provide 'lsp-clients)
 ;;; lsp-clients.el ends here
